@@ -735,6 +735,11 @@ public:
                        get_attribute("DISK_SNAPSHOT_ACTIVE"));
     }
 
+    /**
+     *  @return true if any of the disks has snapshots
+     */
+    bool has_snapshots();
+
     /**
      *  Set the snapshots for a disk
      *    @param id of disk
