@@ -14,11 +14,14 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+require 'time'
+require 'fileutils'
 require 'HostSyncManager'
 require 'one_helper'
 require 'one_helper/onevm_helper'
-require 'rubygems'
-require 'time'
+require 'opennebula/host'
+require 'opennebula/host_pool'
+require 'opennebula/system'
 
 # implements onehost command
 class OneHostHelper < OpenNebulaHelper::OneHelper
@@ -768,11 +771,15 @@ class OneHostHelper < OpenNebulaHelper::OneHelper
     end
 
     def get_numa_cores(cores)
+        return unless cores
+
         ret  = ''
         free = 0
         used = 0
 
         [cores].flatten.each do |info|
+            next unless info['CPUS']
+
             core = info['CPUS'].split(',')
 
             core.uniq! do |c|
