@@ -87,7 +87,7 @@ const AddAddressRangeAction = memo(
 const UpdateAddressRangeAction = memo(
   ({ vnetId, ar, onSubmit, oneConfig, adminGroup }) => {
     const [updateAR] = VnAPI.useUpdateVNetRangeMutation()
-    const { AR_ID } = ar
+    const { AR_ID, LEASES } = ar
 
     const handleUpdate = async (formData) => {
       if (onSubmit && typeof onSubmit === 'function') {
@@ -118,6 +118,7 @@ const UpdateAddressRangeAction = memo(
                 initialValues: ar,
                 stepProps: {
                   isUpdate: !onSubmit && AR_ID !== undefined,
+                  hasLease: typeof LEASES === 'object',
                   oneConfig,
                   adminGroup,
                   restrictedAttributesType: RESTRICTED_ATTRIBUTES_TYPE.VNET,
@@ -133,7 +134,7 @@ const UpdateAddressRangeAction = memo(
 )
 
 const DeleteAddressRangeAction = memo(
-  ({ vnetId, ar, onSubmit, oneConfig, adminGroup }) => {
+  ({ vnetId, ar, onSubmit, oneConfig, adminGroup, submit }) => {
     const [removeAR] = VnAPI.useRemoveRangeFromVNetMutation()
     const { AR_ID } = ar
 
@@ -142,7 +143,10 @@ const DeleteAddressRangeAction = memo(
         return await onSubmit(AR_ID)
       }
 
-      await removeAR({ id: vnetId, address: AR_ID })
+      // When deleting AR in a provision, use the oneform API
+      submit
+        ? await submit({ arId: AR_ID })
+        : await removeAR({ id: vnetId, address: AR_ID })
     }
 
     // Disable action if the disk has a restricted attribute on the template
@@ -181,6 +185,7 @@ const ActionPropTypes = {
   onSubmit: PropTypes.func,
   oneConfig: PropTypes.object,
   adminGroup: PropTypes.bool,
+  submit: PropTypes.object,
 }
 
 AddAddressRangeAction.propTypes = ActionPropTypes
