@@ -23,15 +23,9 @@ import {
   disableFields,
 } from '@UtilsModule'
 import { T, INPUT_TYPES } from '@ConstantsModule'
-import { uniqWith } from 'lodash'
+import { uniqWith, isEqual } from 'lodash'
 import { useFormContext } from 'react-hook-form'
 
-const samePciDevice = (a, b) =>
-  a &&
-  b &&
-  ['DEVICE', 'VENDOR', 'CLASS'].every((k) => a[k] === b[k]) &&
-  a.SHORT_ADDRESS?.split('.')[0] === b.SHORT_ADDRESS?.split('.')[0]
-
 /**
  * Transform a PCI device to String.
  *
@@ -41,7 +35,7 @@ const samePciDevice = (a, b) =>
 export const transformPciToString = (pciDevice = {}) => {
   const { DEVICE = '', VENDOR = '', CLASS = '' } = pciDevice
 
-  return [DEVICE, VENDOR, CLASS].join(';')
+  return [DEVICE, VENDOR, CLASS]?.filter(Boolean)?.join(';') ?? ''
 }
 
 /**
@@ -69,7 +63,6 @@ const SPECIFIC_DEVICE = {
 /** @type {Field} Name PCI device field */
 const NAME_FIELD = {
   name: 'PCI_DEVICE_NAME',
-
   label: T.DeviceName,
   type: INPUT_TYPES.AUTOCOMPLETE,
   optionsOnly: true,
@@ -79,7 +72,18 @@ const NAME_FIELD = {
     const { data: hosts = [] } = HostAPI.useGetHostsAdminQuery()
     const pciDevices = hosts.map(getPciDevices).flat()
 
-    return arrayToOptions(uniqWith(pciDevices, samePciDevice), {
+    const formatPciDevices = []
+      .concat(pciDevices)
+      ?.filter(Boolean)
+      ?.map(({ DEVICE_NAME, TYPE, CLASS, DEVICE, VENDOR }) => ({
+        DEVICE_NAME,
+        TYPE,
+        CLASS,
+        DEVICE,
+        VENDOR,
+      }))
+
+    return arrayToOptions(uniqWith(formatPciDevices, isEqual), {
       getText: ({ DEVICE_NAME } = {}) => DEVICE_NAME,
       addDescription: true,
       getDescription: (opt) => opt?.TYPE,
@@ -186,7 +190,18 @@ const SHORT_ADDRESS = {
     const { data: hosts = [] } = HostAPI.useGetHostsAdminQuery()
     const pciDevices = hosts.map(getPciDevices).flat()
 
-    return arrayToOptions(uniqWith(pciDevices, samePciDevice), {
+    const formatPciDevices = []
+      .concat(pciDevices)
+      ?.filter(Boolean)
+      ?.map(({ DEVICE_NAME, SHORT_ADDRESS: SA, CLASS, DEVICE, VENDOR }) => ({
+        DEVICE_NAME,
+        SHORT_ADDRESS: SA,
+        CLASS,
+        DEVICE,
+        VENDOR,
+      }))
+
+    return arrayToOptions(uniqWith(formatPciDevices, isEqual), {
       addEmpty: false,
       getText: ({ SHORT_ADDRESS: ADDRESS, DEVICE_NAME } = {}) =>
         `${DEVICE_NAME}: ${ADDRESS}`,
@@ -211,19 +226,26 @@ const PROFILE_FIELD = {
   label: T.Profile,
   type: INPUT_TYPES.AUTOCOMPLETE,
   values: (dependencies = []) => {
-    const [selectedPciDevice] = dependencies
+    const [deviceVendorClass = '', shortAddress = '', specificDevice] =
+      dependencies
     const { data = [] } = HostAPI.useGetHostsAdminQuery({
-      skip: selectedPciDevice === undefined,
+      skip: deviceVendorClass === undefined,
     })
-    if (selectedPciDevice && data) {
+    if ((deviceVendorClass || shortAddress) && data) {
       const pciDevices = data.map(getPciDevices).flat()
-      const [DEVICE, VENDOR, CLASS] = selectedPciDevice?.split(';')
-      const selectedDevice = pciDevices.find(
-        (device) =>
-          device?.DEVICE === DEVICE &&
-          device?.VENDOR === VENDOR &&
-          device?.CLASS === CLASS
-      )
+      const selectedDevice = pciDevices.find((device) => {
+        if (specificDevice) {
+          return device?.SHORT_ADDRESS === shortAddress
+        } else {
+          const [DEVICE, VENDOR, CLASS] = deviceVendorClass?.split(';')
+
+          return (
+            device?.DEVICE === DEVICE &&
+            device?.VENDOR === VENDOR &&
+            device?.CLASS === CLASS
+          )
+        }
+      })
 
       const profiles = selectedDevice?.PROFILES?.split(',') || []
 
@@ -237,8 +259,7 @@ const PROFILE_FIELD = {
 
     return arrayToOptions([])
   },
-  dependOf: [NAME_FIELD.name, SPECIFIC_DEVICE.name],
-  htmlType: ([_, specificDevice] = []) => specificDevice && INPUT_TYPES.HIDDEN,
+  dependOf: [NAME_FIELD.name, SHORT_ADDRESS.name, SPECIFIC_DEVICE.name],
   validation: string()
     .trim()
     .notRequired()
