@@ -45,6 +45,7 @@ const ResourcesBackButton = memo(
     info = () => undefined,
     simpleGroupsTags = () => undefined,
     setSelectedRows = () => undefined,
+    customGlobalActions = null,
     actions = [],
     ...restProps
   }) => {
@@ -94,6 +95,7 @@ const ResourcesBackButton = memo(
     const countSelectedRows = selectedRows?.length
     const moreThanOneSelected = countSelectedRows > 1
     const hasSelectedRows = countSelectedRows > 0
+    const hasCustomGlobalActions = typeof customGlobalActions === 'function'
 
     useEffect(() => {
       const viewMode =
@@ -193,10 +195,21 @@ const ResourcesBackButton = memo(
                   condition={showInfo && hasSelectedRows}
                   valid={
                     <>
-                      <GlobalActions
-                        className={styles.actions}
-                        globalActions={actions}
-                        selectedRows={selectedRows}
+                      <Switch
+                        condition={hasCustomGlobalActions}
+                        valid={customGlobalActions?.({
+                          actions,
+                          selectedRows,
+                          setSelectedRows,
+                        })}
+                        invalid={
+                          <GlobalActions
+                            className={styles.actions}
+                            globalActions={actions}
+                            selectedRows={selectedRows}
+                            onSelectedRowsChange={setSelectedRows}
+                          />
+                        }
                       />
                       <Switch
                         condition={moreThanOneSelected}
@@ -307,6 +320,7 @@ ResourcesBackButton.propTypes = {
   simpleGroupsTags: PropTypes.func,
   setSelectedRows: PropTypes.func,
   actions: PropTypes.array,
+  customGlobalActions: PropTypes.func,
 }
 ResourcesBackButton.displayName = 'ResourcesBackButton'
 
