@@ -13,34 +13,36 @@
  * See the License for the specific language governing permissions and       *
  * limitations under the License.                                            *
  * ------------------------------------------------------------------------- */
-import { Typography, useTheme } from '@mui/material'
 import { css } from '@emotion/css'
+import { Typography, useTheme } from '@mui/material'
 import {
-  Plus,
   Cart,
   Group,
   Lock,
-  PlayOutline,
+  Play,
+  Plus,
   SaveFloppyDisk,
+  Settings,
   TransitionRight,
   Trash,
-  Settings,
+  AppleImac2021 as VncIcon,
 } from 'iconoir-react'
 import { useMemo } from 'react'
 import { useHistory } from 'react-router-dom'
 
-import { useViews, useGeneralApi, DatastoreAPI, VmAPI } from '@FeaturesModule'
+import { DatastoreAPI, VmAPI, useGeneralApi, useViews } from '@FeaturesModule'
 
 import {
   GlobalAction,
   createActions,
 } from '@modules/components/Tables/Enhanced/Utils'
 import VmTemplatesTable from '@modules/components/Tables/Vms/VmTemplateTable'
+import VmsConsoleTable from '@modules/components/Tables/Vms/consoleTable'
 
-import { PATH, Form } from '@modules/components'
-import { Translate } from '@modules/components/HOC'
-import { RESOURCE_NAMES, T, VM_ACTIONS, STYLE_BUTTONS } from '@ConstantsModule'
+import { RESOURCE_NAMES, STYLE_BUTTONS, T, VM_ACTIONS } from '@ConstantsModule'
 import { getLastHistory, isVmAvailableAction } from '@ModelsModule'
+import { Form, PATH } from '@modules/components'
+import { Translate } from '@modules/components/HOC'
 const { Vm, Backup } = Form
 
 const useTableStyles = () => ({
@@ -93,12 +95,15 @@ const MessageToConfirmAction = (rows) => (
 /**
  * Generates the actions to operate resources on VM table.
  *
+ * @param {object} props - datatable props
+ * @param {Function} props.setSelectedRows - set selected rows
  * @returns {GlobalAction} - Actions
  */
-const Actions = () => {
+const Actions = (props = {}) => {
+  const { setSelectedRows } = props
   const history = useHistory()
   const { view, getResourceView } = useViews()
-  const { enqueueSuccess } = useGeneralApi()
+  const { enqueueSuccess, setSecondTitle } = useGeneralApi()
 
   const [saveAsTemplate] = VmAPI.useSaveAsTemplateMutation()
   const [actionVm] = VmAPI.useActionVmMutation()
@@ -138,7 +143,10 @@ const Actions = () => {
                     )
 
                     const redirectToInstantiate = (template) =>
-                      history.push(PATH.TEMPLATE.VMS.INSTANTIATE, template)
+                      history.push(PATH.TEMPLATE.VMS.INSTANTIATE, {
+                        ...template,
+                        navigateUrl: 'vm',
+                      })
 
                     return (
                       <VmTemplatesTable
@@ -168,7 +176,7 @@ const Actions = () => {
             size: STYLE_BUTTONS.SIZE.MEDIUM,
             type: STYLE_BUTTONS.TYPE.FILLED,
             selected: true,
-            icon: PlayOutline,
+            icon: Play,
             action: async (rows) => {
               const ids = rows?.map?.(({ original }) => original?.ID)
               await Promise.all(
@@ -533,6 +541,48 @@ const Actions = () => {
               },
             ],
           },
+          {
+            accessor: VM_ACTIONS.VNC,
+            dataCy: 'vm-console-access',
+            tooltip: T.ConsoleAccess,
+            label: T.ConsoleAccess,
+            selected: { min: 1 },
+            icon: VncIcon,
+            importance: STYLE_BUTTONS.IMPORTANCE.SECONDARY,
+            size: STYLE_BUTTONS.SIZE.MEDIUM,
+            type: STYLE_BUTTONS.TYPE.OUTLINED,
+            options: [
+              {
+                isConfirmDialog: true,
+                dialogProps: {
+                  title: T.ConsoleAccess,
+                  children: (rows) => {
+                    const filterVms =
+                      rows?.map((row) => row?.original?.ID).filter(Boolean) ??
+                      []
+                    const theme = useTheme()
+                    const classes = useMemo(
+                      () => useTableStyles(theme),
+                      [theme]
+                    )
+
+                    return (
+                      <VmsConsoleTable
+                        disableGlobalSort
+                        disableRowSelect
+                        filterData={filterVms}
+                        classes={classes}
+                      />
+                    )
+                  },
+                  fixedWidth: true,
+                  fixedHeight: true,
+                  handleAccept: undefined,
+                  dataCy: `modal-${VM_ACTIONS.RDP}`,
+                },
+              },
+            ],
+          },
           {
             accessor: VM_ACTIONS.CREATE_APP_DIALOG,
             dataCy: `vm_${VM_ACTIONS.CREATE_APP_DIALOG}`,
@@ -547,7 +597,7 @@ const Actions = () => {
             action: (rows) => {
               const vm = rows?.[0]?.original ?? {}
               const path = PATH.STORAGE.MARKETPLACE_APPS.CREATE
-
+              setSecondTitle({})
               history.push(path, [RESOURCE_NAMES.VM, vm])
             },
           },
@@ -593,6 +643,7 @@ const Actions = () => {
                   title: T.ChangeOwner,
                   subheader: SubHeader,
                   dataCy: `modal-${VM_ACTIONS.CHANGE_OWNER}`,
+                  validateOn: 'onSubmit',
                 },
                 form: Vm.ChangeUserForm,
                 onSubmit: (rows) => async (newOwnership) => {
@@ -610,6 +661,7 @@ const Actions = () => {
                   title: T.ChangeGroup,
                   subheader: SubHeader,
                   dataCy: `modal-${VM_ACTIONS.CHANGE_GROUP}`,
+                  validateOn: 'onSubmit',
                 },
                 form: Vm.ChangeGroupForm,
                 onSubmit: (rows) => async (newOwnership) => {
@@ -686,6 +738,7 @@ const Actions = () => {
                   await Promise.all(
                     ids.map((id) => actionVm({ id, action: 'terminate' }))
                   )
+                  setSelectedRows && setSelectedRows([])
                 },
               },
               {
@@ -703,6 +756,7 @@ const Actions = () => {
                   await Promise.all(
                     ids.map((id) => actionVm({ id, action: 'terminate-hard' }))
                   )
+                  setSelectedRows && setSelectedRows([])
                 },
               },
             ],
