@@ -15,8 +15,6 @@
  * ------------------------------------------------------------------------- */
 import * as Setting from '@modules/constants/setting'
 
-export const JWT_NAME = 'FireedgeToken'
-
 export const BY = { text: 'by OpenNebula', url: 'https://opennebula.io/' }
 export const SUPPORT_WEBSITE = 'https://opennebula.io/support/'
 export const COMMUNITY_WEBSITE = 'https://opennebula.io/usec'
@@ -72,6 +70,7 @@ export const WEBSOCKET_URL = `${APP_URL}/websockets`
 export const STATIC_FILES_URL = `${APP_URL}/client/assets`
 export const IMAGES_URL = `${STATIC_FILES_URL}/images`
 export const LOGO_IMAGES_URL = `${IMAGES_URL}/logos`
+export const LOGO_DRIVERS_IMAGES_URL = `${IMAGES_URL}/providers`
 export const DEFAULT_IMAGE = `${IMAGES_URL}/default.webp`
 export const IMAGE_FORMATS = ['webp', 'png', 'jpg']
 
@@ -180,12 +179,15 @@ export const RESOURCE_NAMES = {
   BACKUP: 'backup',
   CLUSTER: 'cluster',
   DATASTORE: 'datastore',
+  DRIVER: 'driver',
   GROUP: 'group',
   HOST: 'host',
   IMAGE: 'image',
   FILE: 'file',
   LOGO: 'logo',
   MARKETPLACE: 'marketplace',
+  PROVIDER: 'provider',
+  PROVISION: 'provision',
   SEC_GROUP: 'security-group',
   USER: 'user',
   VDC: 'virtual-data-center',
@@ -213,6 +215,7 @@ export * from '@modules/constants/cluster'
 export * from '@modules/constants/color'
 export * from '@modules/constants/common'
 export * from '@modules/constants/datastore'
+export * from '@modules/constants/driver'
 export * from '@modules/constants/errors'
 export * from '@modules/constants/flow'
 export * from '@modules/constants/group'
@@ -229,6 +232,9 @@ export * from '@modules/constants/scheduler'
 export * from '@modules/constants/securityGroup'
 export * from '@modules/constants/serviceTemplate'
 export * as STATES from '@modules/constants/states'
+export * from '@modules/constants/ods'
+export * from '@modules/constants/provider'
+export * from '@modules/constants/provision'
 export * from '@modules/constants/styles'
 export * from '@modules/constants/support'
 export * from '@modules/constants/system'
