@@ -13,8 +13,7 @@
  * See the License for the specific language governing permissions and       *
  * limitations under the License.                                            *
  * ------------------------------------------------------------------------- */
-import { HostTabs, TranslateProvider } from '@ComponentsModule'
-import { HostsTable } from '@modules/components/Tables'
+import { HostsTable, HostTabs, TranslateProvider } from '@ComponentsModule'
 import { ReactElement } from 'react'
 import { Redirect, useParams } from 'react-router-dom'
 
