@@ -13,9 +13,9 @@
  * See the License for the specific language governing permissions and       *
  * limitations under the License.                                            *
  * ------------------------------------------------------------------------- */
-import _, { cloneDeep, merge, get } from 'lodash'
-import { convertToMB, isBase64, scaleVcpuByCpuFactor } from '@modules/utils'
 import { MEMORY_RESIZE_OPTIONS, T } from '@ConstantsModule'
+import { convertToMB, scaleVcpuByCpuFactor } from '@modules/utils'
+import _, { cloneDeep, get, merge } from 'lodash'
 
 // Attributes that will be always modify with the value of the form (except Storage, Network and PCI sections)
 const alwaysIncludeAttributes = {
@@ -29,6 +29,9 @@ const alwaysIncludeAttributes = {
         BOOT: true,
         FIRMWARE: true,
       },
+      MEMORY_ENCRYPTION: {
+        TYPE: true,
+      },
     },
     InputOutput: {
       INPUT: true,
@@ -73,6 +76,9 @@ const defaultValuesCreate = {
         TYPE: true,
       },
     },
+    OsCpu: {
+      MEMORY_ENCRYPTION: true,
+    },
     Placement: {
       SCHED_REQUIREMENTS: true,
     },
@@ -82,6 +88,7 @@ const defaultValuesCreate = {
 const defaultValuesUpdate = {
   extra: {
     Context: { CONTEXT: { START_SCRIPT: true, START_SCRIPT_BASE64: true } },
+    OsCpu: { MEMORY_ENCRYPTION: true },
   },
 }
 
@@ -134,11 +141,13 @@ const filterTemplateData = (
     ...formData['custom-variables'],
   }
 
-  const result = {
-    ...newGeneral,
-    ...newExtra,
-    ...newCustomVariables,
-  }
+  const result = merge(
+    {},
+    newGeneral,
+    newExtra,
+    newCustomVariables,
+    formData?.user_inputs
+  )
 
   // Instantiate form could have another step called user_inputs
   if (formData.user_inputs) {
@@ -326,8 +335,50 @@ const reduceExtra = (
     }
   )
 
+  const hasMemoryEncryptionField = Object.hasOwn(
+    formData?.extra ?? {},
+    'MEMORY_ENCRYPTION'
+  )
+  const memoryEncryptionTouched =
+    hasMemoryEncryptionField ||
+    Boolean(correctionMap?.extra?.OsCpu?.MEMORY_ENCRYPTION)
+
+  // Check if TYPE is actually set (not empty, undefined, or null)
+  const encryptionTypeValue = formData?.extra?.MEMORY_ENCRYPTION?.TYPE
+  const hasEncryptionType =
+    encryptionTypeValue &&
+    encryptionTypeValue !== '' &&
+    encryptionTypeValue !== 'undefined'
+
+  // If MEMORY_ENCRYPTION was touched but TYPE is cleared/empty, remove the entire block
+  if (memoryEncryptionTouched && !hasEncryptionType) {
+    delete newExtra.MEMORY_ENCRYPTION
+  }
+
   // Omitting values that are empty
-  return _.omitBy(newExtra, _.isEmpty)
+  // But keep MEMORY_ENCRYPTION if TYPE is set
+  const filtered = _.omitBy(newExtra, (value, key) => {
+    if (key === 'MEMORY_ENCRYPTION' && value?.TYPE) {
+      return false // Don't omit MEMORY_ENCRYPTION if TYPE is set
+    }
+
+    return _.isEmpty(value)
+  })
+
+  // Clean up MEMORY_ENCRYPTION: remove undefined/empty fields but keep the section if TYPE exists
+  if (filtered.MEMORY_ENCRYPTION) {
+    filtered.MEMORY_ENCRYPTION = _.omitBy(
+      filtered.MEMORY_ENCRYPTION,
+      (v) => v === undefined || v === null || v === ''
+    )
+
+    // If after cleanup there's no TYPE, remove the whole section
+    if (!filtered.MEMORY_ENCRYPTION?.TYPE) {
+      delete filtered.MEMORY_ENCRYPTION
+    }
+  }
+
+  return filtered
 }
 
 /**
@@ -595,6 +646,15 @@ const handleOtherSections = (
           originalData,
           alwaysIncludePci
         )
+      } else if (key === 'MEMORY_ENCRYPTION') {
+        // Special handling for MEMORY_ENCRYPTION: replace entirely with form values when present
+        if (formData?.extra?.MEMORY_ENCRYPTION) {
+          newExtra[key] = { ...formData.extra[key] }
+        }
+        // If form doesn't have MEMORY_ENCRYPTION but corrections do, process normally
+        else if (correctionMap.extra[section]?.[key]) {
+          filterSingleSection(formData, correctionMap, section, newExtra, key)
+        }
       } else {
         filterSingleSection(formData, correctionMap, section, newExtra, key)
 
@@ -689,14 +749,6 @@ const filterSingleSection = (
 const transformActionsCreate = (template) => {
   transformActionsCommon(template)
 
-  // Encode script on base 64, if needed, on context section
-  if (isBase64(template?.CONTEXT?.START_SCRIPT)) {
-    template.CONTEXT.START_SCRIPT_BASE64 = template?.CONTEXT?.START_SCRIPT
-    delete template?.CONTEXT?.START_SCRIPT
-  } else {
-    delete template?.CONTEXT?.START_SCRIPT_BASE64
-  }
-
   if (template.RAW) {
     // Clone template.RAW to ensure its mutable
     template.RAW = { ...template.RAW }
