@@ -29,6 +29,7 @@ import {
   endpointExternalGuacamole,
 } from './utils/constants/defaults'
 import { getLoggerMiddleware, initLogger } from './utils/logger'
+import { ensureSessionStore } from './utils/sessions'
 import {
   genFireedgeKey,
   genPathResources,
@@ -48,6 +49,8 @@ import opennebulaWebsockets from './routes/websockets/opennebula'
 import { messageTerminal } from './utils/general'
 import { getFireedgeConfig } from './utils/yml'
 
+ensureSessionStore()
+
 setDnsResultOrder()
 
 // set paths
