@@ -15,12 +15,24 @@
  * ------------------------------------------------------------------------- */
 
 const { Actions, Commands } = require('server/routes/api/auth/routes')
-const { selectTypeAuth } = require('server/routes/api/auth/functions')
-const { AUTHENTICATION } = Actions
+const {
+  selectTypeAuth,
+  samlAuth,
+  logout,
+} = require('server/routes/api/auth/functions')
+const { AUTHENTICATION, LOGOUT, SAML_AUTHENTICATION } = Actions
 
 module.exports = [
   {
     ...Commands[AUTHENTICATION],
     action: selectTypeAuth,
   },
+  {
+    ...Commands[LOGOUT],
+    action: logout,
+  },
+  {
+    ...Commands[SAML_AUTHENTICATION],
+    action: samlAuth,
+  },
 ]
