@@ -24,11 +24,13 @@ const { postBody } = fromData
 
 const basepath = '/auth'
 const AUTHENTICATION = 'authentication'
+const LOGOUT = 'logout'
 const SAML_AUTHENTICATION = 'authentication.saml'
 
 const Actions = {
   AUTHENTICATION,
   SAML_AUTHENTICATION,
+  LOGOUT,
 }
 
 module.exports = {
@@ -48,7 +50,7 @@ module.exports = {
         type: {
           from: postBody,
         },
-        token2fa: {
+        tfatoken: {
           from: postBody,
         },
         remember: {
@@ -56,6 +58,11 @@ module.exports = {
         },
       },
     },
+    [LOGOUT]: {
+      path: `${basepath}/logout`,
+      httpMethod: POST,
+      auth: true,
+    },
     [SAML_AUTHENTICATION]: {
       path: `${basepath}/acs`,
       httpMethod: POST,
