@@ -21,12 +21,14 @@
 #include <stdexcept>
 #include <iomanip>
 #include <set>
+#include <map>
 
 #include <math.h>
 
 #include "HostSharePCI.h"
 #include "Host.h"
 #include "HostShare.h"
+#include "Nebula.h"
 
 using namespace std;
 
@@ -198,7 +200,7 @@ void HostSharePCI::pci_attribute(VectorAttribute *device, PCIDevice *pci,
                                  bool set_prev, const std::string& vprofile)
 {
     static vector<string> cp_attr = {"DOMAIN", "BUS", "SLOT", "FUNCTION", "ADDRESS",
-                                     "SHORT_ADDRESS"};
+                                     "SHORT_ADDRESS", "VENDOR", "DEVICE", "CLASS"};
 
     static vector<string> cp_check_attr = {"NUMA_NODE", "UUID", "MDEV_MODE"};
 
@@ -248,7 +250,12 @@ void HostSharePCI::pci_attribute(VectorAttribute *device, PCIDevice *pci,
         // NVIDIA Corporation && 3D controller
         if ((vendor_id == 0x10de) && (class_id == 0x0302))
         {
-            device->replace("PROFILE", vprofile);
+            const std::string& dev_profile = device->vector_value("PROFILE");
+
+            if (dev_profile.empty())
+            {
+                device->replace("PROFILE", vprofile);
+            }
         }
     }
 }
@@ -256,7 +263,8 @@ void HostSharePCI::pci_attribute(VectorAttribute *device, PCIDevice *pci,
 /* -------------------------------------------------------------------------- */
 
 bool HostSharePCI::add_by_addr(VectorAttribute *device, const string& short_addr,
-                               int vmid, const std::string& vprofile)
+                               std::map<unsigned int, std::set<unsigned int>>& palloc,
+                               HostShareCapacity &sr)
 {
     for (auto jt = pci_devices.begin(); jt != pci_devices.end(); jt++)
     {
@@ -272,11 +280,18 @@ bool HostSharePCI::add_by_addr(VectorAttribute *device, const string& short_addr
             return false;
         }
 
-        pci->vmid = vmid;
+        pci->vmid = sr.vmid;
 
-        pci->attrs->replace("VMID", vmid);
+        pci->attrs->replace("VMID", sr.vmid);
 
-        pci_attribute(device, pci, true, vprofile);
+        pci_attribute(device, pci, true, sr.vgpu_profile);
+
+        if (set_pci_address(device, palloc, sr.is_q35, sr.nodes) == -1)
+        {
+            pci->attrs->replace("VMID", "-1");
+
+            return false;
+        }
 
         return true;
     }
@@ -286,7 +301,9 @@ bool HostSharePCI::add_by_addr(VectorAttribute *device, const string& short_addr
 
 /* -------------------------------------------------------------------------- */
 
-bool HostSharePCI::add_by_name(VectorAttribute *device, int vmid, const std::string& vprofile)
+bool HostSharePCI::add_by_name(VectorAttribute *device,
+                               std::map<unsigned int, std::set<unsigned int>>& palloc,
+                               HostShareCapacity &sr)
 {
     unsigned int vendor_id, device_id, class_id;
 
@@ -308,11 +325,18 @@ bool HostSharePCI::add_by_name(VectorAttribute *device, int vmid, const std::str
             (device_rc == 0 || pci->device_id == device_id) &&
             pci->vmid  == -1 )
         {
-            pci->vmid = vmid;
+            pci->vmid = sr.vmid;
 
-            pci->attrs->replace("VMID", vmid);
+            pci->attrs->replace("VMID", sr.vmid);
 
-            pci_attribute(device, pci, true, vprofile);
+            pci_attribute(device, pci, true, sr.vgpu_profile);
+
+            if (set_pci_address(device, palloc, sr.is_q35, sr.nodes) == -1)
+            {
+                pci->attrs->replace("VMID", "-1");
+
+                return false;
+            }
 
             return true;
         }
@@ -323,11 +347,15 @@ bool HostSharePCI::add_by_name(VectorAttribute *device, int vmid, const std::str
 
 /* -------------------------------------------------------------------------- */
 
-bool HostSharePCI::add(vector<VectorAttribute *> &devs, int vmid, const std::string& vprofile)
+bool HostSharePCI::add(HostShareCapacity &sr)
 {
     std::set<VectorAttribute *> added;
     unsigned int vendor_id, device_id, class_id;
 
+    vector<VectorAttribute *> &devs = sr.pci;
+
+    std::map<unsigned int, std::set<unsigned int>> palloc;
+
     for (auto& device : devs)
     {
         string short_addr = device->vector_value("SHORT_ADDRESS");
@@ -340,7 +368,7 @@ bool HostSharePCI::add(vector<VectorAttribute *> &devs, int vmid, const std::str
             continue;
         }
 
-        if (!add_by_addr(device, short_addr, vmid, vprofile))
+        if (!add_by_addr(device, short_addr, palloc, sr))
         {
             return false;
         }
@@ -355,7 +383,7 @@ bool HostSharePCI::add(vector<VectorAttribute *> &devs, int vmid, const std::str
             continue;
         }
 
-        if (!add_by_name(device, vmid, vprofile))
+        if (!add_by_name(device, palloc, sr))
         {
             return false;
         }
@@ -367,8 +395,12 @@ bool HostSharePCI::add(vector<VectorAttribute *> &devs, int vmid, const std::str
 /* ------------------------------------------------------------------------*/
 /* ------------------------------------------------------------------------*/
 
-void HostSharePCI::del(const vector<VectorAttribute *> &devs, int vmid)
+void HostSharePCI::del(HostShareCapacity &sr)
 {
+    const vector<VectorAttribute *> &devs = sr.pci;
+
+    int vmid = sr.vmid;
+
     for (auto device : devs)
     {
         auto pci_it = pci_devices.find(device->vector_value("PREV_ADDRESS"));
@@ -487,6 +519,7 @@ void HostSharePCI::set_monitorization(Template& ht, const HostShareConf& hconf)
         while (getline(iss, filter_str, ','))
         {
             filters.emplace_back(std::array<std::string, 3>{"*", "*", "*"});
+
             auto& filter = filters.back();
 
             std::istringstream filter_stream(filter_str);
@@ -532,8 +565,16 @@ void HostSharePCI::set_monitorization(Template& ht, const HostShareConf& hconf)
         if (pci_it != pci_devices.end())
         {
             missing.erase(address);
-            delete pci;
 
+            const auto& stored_profiles    = pci_it->second->attrs->vector_value("PROFILES");
+            const auto& monitored_profiles = pci->vector_value("PROFILES");
+
+            if (monitored_profiles != stored_profiles)
+            {
+                pci_it->second->attrs->replace("PROFILES", monitored_profiles);
+            }
+
+            delete pci;
             continue;
         }
 
@@ -606,80 +647,190 @@ int HostSharePCI::get_pci_value(const char * name,
 
 /* ------------------------------------------------------------------------*/
 /* ------------------------------------------------------------------------*/
-
 int HostSharePCI::set_pci_address(VectorAttribute * pci_device,
-                                  const string& dbus, bool bus_index, bool clean)
+        std::map<unsigned int, std::set<unsigned int>>& palloc,
+        bool bus_index, const std::vector<VectorAttribute *> &nodes)
 {
-    string        bus;
     ostringstream oss;
 
-    unsigned int ibus, slot;
+    unsigned int slot;
+    unsigned int total_nodes = 0;
 
-    // ------------------- Remove well-known attributes -----------------------
-    static vector<string> rm_attr = {"DOMAIN", "BUS", "SLOT", "FUNCTION",
-                                     "ADDRESS", "PREV_ADDRESS", "NUMA_NODE",
-                                     "UUID", "MDEV_MODE"
-                                    };
-
-    if (clean)
+    if (pci_device->vector_value("PCI_ID", slot) == -1)
     {
-        for (const auto& attr : rm_attr)
+        return 0; //Attach operation will set PCI_ID after allocation
+    }
+
+    int numa_node = -1;
+
+    if (!nodes.empty())
+    {
+        if (pci_device->vector_value("NUMA_NODE", numa_node) == -1)
         {
-            pci_device->remove(attr);
+            numa_node = -1;
+        }
+
+        for (const auto &n : nodes)
+        {
+            int cpu = 0;
+            int mem = 0;
+
+            n->vector_value("TOTAL_CPUS", cpu);
+            n->vector_value("MEMORY", mem);
+
+            if ( cpu > 0 && mem > 0 ) ++total_nodes;
         }
     }
 
-    // ------------------- DOMAIN & FUNCTION -------------------------
-    pci_device->replace("VM_DOMAIN", "0x0000");
-    pci_device->replace("VM_FUNCTION", "0");
-
-    // --------------------------- BUS -------------------------------
-    bus = pci_device->vector_value("VM_BUS");
-
-    if ( bus.empty() )
+    if (numa_node != -1)
     {
-        bus = dbus;
+        /* ---------------------------------------------------------------------
+         * NUMA node allocation for VM pinning
+         * ---------------------------------------------------------------------
+         *           +-----------+
+         *           |           |
+         *           | PCIe Root |
+         *           |           |
+         *           +--+--+-----+
+         *              |  |
+         *        +-----+  +-------------------------------+
+         *        | NUMA NODE = 0                          | NUMA NODE = 1
+         *   +----------+                             +----------+
+         *   | expander |  index = 3 + (node * 9)     | expander |
+         *   +----------+                             +----------+
+         *        |------> root-port (0) (index+1)         |-------> root-port
+         *        |        ...                             |
+         *        +------> root-port (7) (index+7)         +-------> root-port
+         *        .                                        .
+         *        .                                        .
+         *   +----------+   --- (DEIDCATED DEVs) ---  +----------+
+         *   | expander |-->root-->device             | expander |-->root-->device
+         *   +----------+   index_m+1                 +----------+
+         *      index_m
+         * ------------------------------------------------------------------ */
+
+        string root = pci_device->vector_value("ROOT");
+        bool dedicated = !root.empty() && root == "dedicated";
+
+        unsigned int palloc_idx;
+
+        if (dedicated)
+        {
+            palloc_idx = std::numeric_limits<unsigned int>::max();
+        }
+        else
+        {
+            palloc_idx = static_cast<unsigned int>(numa_node);
+        }
+
+        std::set<unsigned int>& ports = palloc[palloc_idx];
+
+        unsigned int first_port, end_port, port_inc;
+        unsigned int bus_addr, bus_inc;
+
+        unsigned int index   = 0;
+
+        if (dedicated)
+        {
+            //Up to 8 dedicated
+            first_port = 3 + total_nodes * 9 + 1;
+            end_port   = first_port + 24;
+            port_inc   = 2;
+
+            bus_addr   = 0xA0 + 1;
+            bus_inc    = 0x08;
+        }
+        else
+        {
+            first_port = 3 + numa_node * 9 + 1;
+            end_port   = first_port + 8;
+            port_inc   = 1;
+
+            bus_addr   = (0x20 + 0x10 * numa_node) + 1;
+            bus_inc    = 1;
+        }
+
+        for (unsigned int p = first_port ; p < end_port ; p += port_inc)
+        {
+            if (ports.find(p) == ports.end())
+            {
+                index = p;
+                break;
+            }
+
+            bus_addr += bus_inc;
+        }
+
+        if ( index == 0 ) //No free root ports
+        {
+            return -1;
+        }
+
+        ports.insert(index);
+
+        pci_device->replace("VM_BUS_INDEX", index);
+
+        oss << noshowbase << internal << hex << setfill('0') << setw(2) << bus_addr
+            << ":00.0";
+
+        pci_device->replace("VM_ADDRESS", oss.str());
     }
-
-    istringstream iss(bus);
-
-    iss >> hex >> ibus;
-
-    if (iss.fail() || !iss.eof())
+    else
     {
-        return -1;
+        /* ---------------------------------------------------------------------
+         * Flat topology bus allocation
+         * ---------------------------------------------------------------------
+         * - q35 machine PCI devices are attached to pcie-root-ports using a
+         *   different bus based on the PCI_ID
+         * - On non q35 machines each PCI device is attached in a different slot
+         */
+        pci_device->remove("VM_BUS_INDEX");
+
+        // DOMAIN & FUNCTION
+        pci_device->replace("VM_DOMAIN", "0x0000");
+        pci_device->replace("VM_FUNCTION", "0");
+
+        // BUS
+        unsigned int ibus;
+
+        string bus = pci_device->vector_value("VM_BUS");
+
+        istringstream iss(bus);
+
+        iss >> hex >> ibus;
+
+        if (iss.fail() || !iss.eof())
+        {
+            return -1;
+        }
+
+        slot = slot + 1; // Bus slot = (PCI_ID +1)
+
+        if ( bus_index )
+        {
+            ibus = slot;
+            slot = 0;
+        }
+
+        // Set PCI attributes: VM_SLOT, VM_BUS, VM_ADDRESS
+        oss << showbase << internal << hex << setfill('0') << setw(4) << slot;
+
+        pci_device->replace("VM_SLOT", oss.str());
+
+        oss.str("");
+
+        oss << setfill('0') << setw(4) << ibus;
+
+        pci_device->replace("VM_BUS", oss.str());
+
+        oss.str("");
+
+        oss << noshowbase << internal << hex << setfill('0') << setw(2)
+            << ibus << ":" << setfill('0') << setw(2) << slot << ".0";
+
+        pci_device->replace("VM_ADDRESS", oss.str()); //Bus address = BUS:SLOT.0
     }
 
-    // --------------------- SLOT (PCI_ID +1) -----------------------
-    pci_device->vector_value("PCI_ID", slot);
-
-    slot = slot + 1;
-
-    if ( bus_index )
-    {
-        ibus = slot;
-        slot = 0;
-    }
-
-    // Set PCI attributes
-    oss << showbase << internal << setfill('0') << hex << setw(4) << slot;
-
-    pci_device->replace("VM_SLOT", oss.str());
-
-    oss.str("");
-
-    oss << showbase << internal << setfill('0') << hex << setw(4) << ibus;
-
-    pci_device->replace("VM_BUS", oss.str());
-
-    // ------------------- ADDRESS (BUS:SLOT.0) ---------------------
-    oss.str("");
-
-    oss << noshowbase<<internal<<hex<<setfill('0')<<setw(2) << ibus << ":"
-        << noshowbase<<internal<<hex<<setfill('0')<<setw(2) << slot << ".0";
-
-    pci_device->replace("VM_ADDRESS", oss.str());
-
     return 0;
 }
 
