@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2019-2023, OpenNebula Systems S.L.                               #
+# Copyright 2019-2025, OpenNebula Systems S.L.                               #
 #                                                                            #
 # Licensed under the OpenNebula Software License                             #
 # (the "License"); you may not use this file except in compliance with       #
@@ -16,17 +16,15 @@
 # -------------------------------------------------------------------------- #
 
 module Migrator
-
     def db_version
-        '6.0.0'
+        "5.6.0"
     end
 
     def one_version
-        'OpenNebula 6.0.0'
+        "OpenNebula 5.6.0"
     end
 
     def up
-        true
+        return true
     end
-
 end
