@@ -247,7 +247,9 @@ module Aws
           {
             'ACM' => 'acm',
             'ACMPCA' => 'acm-pca',
+            'AIOps' => 'aiops',
             'APIGateway' => 'apigateway',
+            'ARCRegionswitch' => 'arc-region-switch',
             'ARCZonalShift' => 'arc-zonal-shift',
             'AccessAnalyzer' => 'access-analyzer',
             'Account' => 'account',
@@ -265,7 +267,6 @@ module Aws
             'AppRunner' => 'apprunner',
             'AppStream' => 'appstream2',
             'AppSync' => 'appsync',
-            'AppTest' => 'apptest',
             'Appflow' => 'appflow',
             'ApplicationAutoScaling' => 'application-autoscaling',
             'ApplicationCostProfiler' => 'application-cost-profiler',
@@ -279,14 +280,18 @@ module Aws
             'AutoScaling' => 'autoscaling',
             'AutoScalingPlans' => 'autoscaling-plans',
             'B2bi' => 'b2bi',
+            'BCMDashboards' => 'bcm-dashboards',
             'BCMDataExports' => 'bcm-data-exports',
             'BCMPricingCalculator' => 'bcm-pricing-calculator',
+            'BCMRecommendedActions' => 'bcm-recommended-actions',
             'Backup' => 'backup',
             'BackupGateway' => 'backup-gateway',
             'BackupSearch' => 'backup-search',
             'Batch' => 'batch',
             'Bedrock' => 'bedrock',
             'BedrockAgent' => 'bedrock-agent',
+            'BedrockAgentCore' => 'bedrock-agentcore',
+            'BedrockAgentCoreControl' => 'bedrock-agentcore-control',
             'BedrockAgentRuntime' => 'bedrock-agent-runtime',
             'BedrockDataAutomation' => 'bedrock-data-automation',
             'BedrockDataAutomationRuntime' => 'bedrock-data-automation-runtime',
@@ -317,7 +322,6 @@ module Aws
             'CloudTrailData' => 'cloudtrail-data',
             'CloudWatch' => 'monitoring',
             'CloudWatchEvents' => 'events',
-            'CloudWatchEvidently' => 'evidently',
             'CloudWatchLogs' => 'logs',
             'CloudWatchRUM' => 'rum',
             'CodeArtifact' => 'codeartifact',
@@ -338,6 +342,7 @@ module Aws
             'Comprehend' => 'comprehend',
             'ComprehendMedical' => 'comprehendmedical',
             'ComputeOptimizer' => 'compute-optimizer',
+            'ComputeOptimizerAutomation' => 'aco-automation',
             'ConfigService' => 'config',
             'Connect' => 'connect',
             'ConnectCampaignService' => 'connect-campaigns',
@@ -388,10 +393,10 @@ module Aws
             'ElasticBeanstalk' => 'elasticbeanstalk',
             'ElasticLoadBalancing' => 'elasticloadbalancing',
             'ElasticLoadBalancingV2' => 'elasticloadbalancing',
-            'ElasticTranscoder' => 'elastictranscoder',
             'ElasticsearchService' => 'es',
             'EntityResolution' => 'entityresolution',
             'EventBridge' => 'events',
+            'Evs' => 'evs',
             'FIS' => 'fis',
             'FMS' => 'fms',
             'FSx' => 'fsx',
@@ -429,11 +434,9 @@ module Aws
             'InternetMonitor' => 'internetmonitor',
             'Invoicing' => 'invoicing',
             'IoT' => 'iot',
-            'IoTAnalytics' => 'iotanalytics',
             'IoTDeviceAdvisor' => 'api.iotdeviceadvisor',
             'IoTEvents' => 'iotevents',
             'IoTEventsData' => 'data.iotevents',
-            'IoTFleetHub' => 'api.fleethub.iot',
             'IoTFleetWise' => 'iotfleetwise',
             'IoTJobsDataPlane' => 'data.jobs.iot',
             'IoTManagedIntegrations' => 'api.iotmanagedintegrations',
@@ -449,6 +452,7 @@ module Aws
             'Kendra' => 'kendra',
             'KendraRanking' => 'kendra-ranking',
             'Keyspaces' => 'cassandra',
+            'KeyspacesStreams' => 'cassandra-streams',
             'Kinesis' => 'kinesis',
             'KinesisAnalytics' => 'kinesisanalytics',
             'KinesisAnalyticsV2' => 'kinesisanalytics',
@@ -470,11 +474,11 @@ module Aws
             'Lightsail' => 'lightsail',
             'LocationService' => 'geo',
             'LookoutEquipment' => 'lookoutequipment',
-            'LookoutMetrics' => 'lookoutmetrics',
-            'LookoutforVision' => 'lookoutvision',
+            'MPA' => 'mpa',
             'MQ' => 'mq',
             'MTurk' => 'mturk-requester',
             'MWAA' => 'airflow',
+            'MWAAServerless' => 'airflow-serverless',
             'MachineLearning' => 'machinelearning',
             'Macie2' => 'macie2',
             'MailManager' => 'mail-manager',
@@ -515,19 +519,22 @@ module Aws
             'NetworkMonitor' => 'networkmonitor',
             'Notifications' => 'notifications',
             'NotificationsContacts' => 'notifications-contacts',
+            'NovaAct' => 'nova-act',
             'OAM' => 'oam',
             'OSIS' => 'osis',
             'ObservabilityAdmin' => 'observabilityadmin',
+            'Odb' => 'odb',
             'Omics' => 'omics',
             'OpenSearchServerless' => 'aoss',
             'OpenSearchService' => 'es',
-            'OpsWorks' => 'opsworks',
-            'OpsWorksCM' => 'opsworks-cm',
             'Organizations' => 'organizations',
             'Outposts' => 'outposts',
             'PCS' => 'pcs',
             'PI' => 'pi',
             'Panorama' => 'panorama',
+            'PartnerCentralAccount' => 'partnercentral-account',
+            'PartnerCentralBenefits' => 'partnercentral-benefits',
+            'PartnerCentralChannel' => 'partnercentral-channel',
             'PartnerCentralSelling' => 'partnercentral-selling',
             'PaymentCryptography' => 'controlplane.payment-cryptography',
             'PaymentCryptographyData' => 'dataplane.payment-cryptography',
@@ -548,12 +555,11 @@ module Aws
             'QApps' => 'data.qapps',
             'QBusiness' => 'qbusiness',
             'QConnect' => 'wisdom',
-            'QLDB' => 'qldb',
-            'QLDBSession' => 'session.qldb',
             'QuickSight' => 'quicksight',
             'RAM' => 'ram',
             'RDS' => 'rds',
             'RDSDataService' => 'rds-data',
+            'RTBFabric' => 'rtbfabric',
             'RecycleBin' => 'rbin',
             'Redshift' => 'redshift',
             'RedshiftDataAPIService' => 'redshift-data',
@@ -564,10 +570,10 @@ module Aws
             'ResourceExplorer2' => 'resource-explorer-2',
             'ResourceGroups' => 'resource-groups',
             'ResourceGroupsTaggingAPI' => 'tagging',
-            'RoboMaker' => 'robomaker',
             'RolesAnywhere' => 'rolesanywhere',
             'Route53' => 'route53',
             'Route53Domains' => 'route53domains',
+            'Route53GlobalResolver' => 'route53globalresolver',
             'Route53Profiles' => 'route53profiles',
             'Route53RecoveryCluster' => 'route53-recovery-cluster',
             'Route53RecoveryControlConfig' => 'route53-recovery-control-config',
@@ -577,9 +583,9 @@ module Aws
             'S3Control' => 's3-control',
             'S3Outposts' => 's3-outposts',
             'S3Tables' => 's3tables',
+            'S3Vectors' => 's3vectors',
             'SES' => 'email',
             'SESV2' => 'email',
-            'SMS' => 'sms',
             'SNS' => 'sns',
             'SQS' => 'sqs',
             'SSM' => 'ssm',
@@ -597,6 +603,7 @@ module Aws
             'SageMakerGeospatial' => 'sagemaker-geospatial',
             'SageMakerMetrics' => 'metrics.sagemaker',
             'SageMakerRuntime' => 'runtime.sagemaker',
+            'SageMakerRuntimeHTTP2' => 'runtime.sagemaker',
             'SagemakerEdgeManager' => 'edge.sagemaker',
             'SavingsPlans' => 'savingsplans',
             'Scheduler' => 'scheduler',
@@ -611,6 +618,8 @@ module Aws
             'ServiceQuotas' => 'servicequotas',
             'Shield' => 'shield',
             'Signer' => 'signer',
+            'SignerData' => 'data-signer',
+            'Signin' => 'signin',
             'SimSpaceWeaver' => 'simspaceweaver',
             'SimpleDB' => 'sdb',
             'SnowDeviceManagement' => 'snow-device-management',
@@ -641,12 +650,14 @@ module Aws
             'WAFRegional' => 'waf-regional',
             'WAFV2' => 'wafv2',
             'WellArchitected' => 'wellarchitected',
+            'Wickr' => 'admin.wickr',
             'WorkDocs' => 'workdocs',
             'WorkMail' => 'workmail',
             'WorkMailMessageFlow' => 'workmailmessageflow',
             'WorkSpaces' => 'workspaces',
             'WorkSpacesThinClient' => 'thinclient',
             'WorkSpacesWeb' => 'workspaces-web',
+            'WorkspacesInstances' => 'workspaces-instances',
             'XRay' => 'xray',
           }
           # end service ids
