@@ -17,7 +17,11 @@
 #--------------------------------------------------------------------------- #
 
 require 'CommandManager'
+require 'rexml/document'
+
 require_relative 'kvm'
+require_relative 'shell'
+require_relative 'backup_qcow2'
 
 module TransferManager
 
@@ -26,25 +30,56 @@ module TransferManager
 
         include TransferManager::KVM
 
-        def initialize(vm_xml, disks)
-            @xml   = vm_xml
-            @disks = disks
+        def initialize(vm_xml, vm_dir, disks)
+            @xml    = vm_xml
+            @vm_dir = vm_dir
+            @disks  = disks
         end
 
-        def backup_disks_sh(disks, backup_dir, ds, live, deploy_id = nil)
+        def backup_disks_sh(options = {})
+            disks       = options[:disks]
+            backup_dir  = options[:backup_dir]
+            ds          = options[:ds]
+            live        = options[:live]
+            deploy_id   = options[:deploy_id]
+            bridge_host = options[:bridge_host]
+
+            kvm = KVMDomain.new(@xml, @vm_dir, :backup_dir => backup_dir)
+
             snap_cmd = ''
             expo_cmd = ''
-            clup_cmd = ''
+
+            snap_clup = ''
+            expo_clup = ''
+
+            xml_vm = REXML::Document.new(@xml).root
+
+            bk_img_id_elem = xml_vm.elements['BACKUPS/BACKUP_IDS/ID']
+            bk_img_id      = bk_img_id_elem&.text&.strip
+
+            if bk_img_id.nil? || bk_img_id.empty?
+                disk_format = 'qcow2'
+            else
+                client = OpenNebula::Client.new
+                bk_img = OpenNebula::Image.new_with_id(bk_img_id, client)
+
+                bk_img.info
+
+                disk_format = bk_img['FORMAT']&.strip
+            end
+
             @disks.compact.each do |d|
                 did = d.id
                 next unless disks.include? did.to_s
 
-                cmds = d.backup_cmds(backup_dir, ds, live)
+                # Pass the format for this disk (or nil if not set)
+                cmds = d.backup_cmds(backup_dir, ds, live, disk_format)
                 return nil unless cmds
 
-                snap_cmd << cmds[:snapshot]
-                expo_cmd << cmds[:export]
-                clup_cmd << cmds[:cleanup]
+                snap_cmd  << cmds[:snapshot].to_s
+                expo_cmd  << cmds[:export].to_s
+                snap_clup << cmds[:snapshot_clup].to_s
+                expo_clup << cmds[:export_clup].to_s
             end
 
             freeze, thaw =
@@ -54,7 +89,7 @@ module TransferManager
                     ['', '']
                 end
 
-            <<~EOS
+            eos1 = <<~EOS1
                 set -ex -o pipefail
 
                 # ----------------------
@@ -67,21 +102,49 @@ module TransferManager
                 echo "#{Base64.encode64(@xml)}" > #{backup_dir}/vm.xml
 
                 # --------------------------------
-                # Create LVM snapshots for disks
+                # Create snapshots for disks
                 # --------------------------------
                 #{freeze}
 
                 #{snap_cmd}
 
                 #{thaw}
+            EOS1
+
+            eos2 = <<~EOS2
+                set -ex -o pipefail
+
+                # --------------------------------------
+                # Save TPM state
+                # --------------------------------------
+                #{kvm.save_tpm_sh}
 
                 # --------------------------
-                # export, convert & cleanup
+                # Export, convert & cleanup
                 # --------------------------
+                [ -d #{backup_dir} ] || mkdir -p #{backup_dir}
+
                 #{expo_cmd}
 
-                #{clup_cmd}
-            EOS
+                cd #{backup_dir}
+
+                #{expo_clup}
+            EOS2
+
+            eos3 = <<~EOS3
+                set -ex -o pipefail
+
+                # --------------------------
+                # Cleanup snapshots
+                # --------------------------
+                #{snap_clup}
+            EOS3
+
+            [
+                [nil, eos1],
+                [bridge_host, eos2],
+                [nil, eos3]
+            ].map {|(host, cmd)| TransferManager::Shell.sshwrap(host, cmd) }.join("\n\n")
         end
 
     end
