@@ -13,11 +13,12 @@
 # See the License for the specific language governing permissions and        #
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
-require_relative '../lib/xmlparser'
+
 require_relative '../lib/command'
 require_relative '../lib/opennebula_vm'
 
 require 'json'
+require 'tempfile'
 
 # rubocop:disable Style/ClassAndModuleChildren
 # rubocop:disable Style/ClassVars
@@ -50,7 +51,7 @@ module VirtualMachineManagerKVM
         File.readlines(path).each do |l|
             next if l.empty? || l[0] == '#'
 
-            m = l.match(/(export)?[[:blank:]]*([^=]+)=([^[[:blank:]]]+)$/)
+            m = l.match(/(export)?[[:blank:]]*([^=]+)=([[:blank:]]*(['"]).*?\4|[^#\n]*)/)
 
             next unless m
 
@@ -168,6 +169,8 @@ module VirtualMachineManagerKVM
     #
     class KvmDomain
 
+        attr_reader :domain
+
         def initialize(domain)
             @domain = domain
             @xml    = nil
@@ -272,8 +275,13 @@ module VirtualMachineManagerKVM
                     raise StandardError, "Error getting domain snapshots #{e}"
                 end
 
-                @disks = o.lines[2..o.lines.length].map {|l| l.split }
-                @disks.reject! {|s| s.empty? }
+                @disks =
+                    o.lines[2..o.lines.length]
+                     .reject {|l| l.chomp.empty? }
+                     .map do |l|
+                         (dev, path) = l.split
+                         [dev, Pathname.new(path)]
+                     end
             end
 
             @disks
@@ -311,6 +319,65 @@ module VirtualMachineManagerKVM
             virsh_retry(cmd, 'active block job', virsh_tries)
         end
 
+        # Live migrate the given disks between the given VM directories.
+        # This only works within the same host.
+        #
+        #   @param disks[Array] disks (as (dev,path) tuples) to be moved
+        #          Example: [['vda', '/var/lib/one/datastores/0/12/disk.0']]
+        #   @param rodisks[Array] read-only disks (as (dev,path) tuples) to be redefined
+        #          Example: [['hda', '/var/lib/one/datastores/0/12/disk.1']]
+        #   @param src_dir[Pathname] source VM dir
+        #          Example: /var/lib/one/datastores/0/12
+        #   @param dst_dir[Pathname] destination VM dir
+        #          Example: /var/lib/one/datastores/100/12
+        def live_blockcopy_disks(disks, rodisks, src_dir, dst_dir)
+            disks.each do |(dev, path)|
+                new_path = dst_dir + path.relative_path_from(src_dir)
+
+                `mkdir -p #{new_path.dirname}; touch #{new_path}`
+
+                qimg = QemuImg.new(path.to_s)
+                has_backing = qimg['backing-filename'] && !qimg['backing-filename'].empty?
+
+                # --pivot: Pivot the VM to use the new disk after copy.
+                # --reuse-external: Reuse existing backing files on the destination.
+                # --shallow: Only copy the top layer of a disk with backing files.
+                # --blockdev: Destination file is a block device (e.g., LV)
+                cmd =  "blockcopy #{@domain} #{dev} #{new_path} " \
+                       '--wait --verbose --pivot --reuse-external'
+                cmd << ' --shallow' if has_backing
+                cmd << ' --blockdev' if path.symlink? && path.readlink.blockdev?
+
+                rc, out, err = virsh_retry(cmd, 'active block job', virsh_tries)
+                return [rc, out, err] if rc != 0
+            end
+
+            rodisks.each do |(dev, path)|
+                new_path = dst_dir + path.relative_path_from(src_dir)
+                cmd      = "change-media #{@domain} #{dev} #{new_path} --update --force"
+
+                rc, out, err = virsh_retry(cmd, 'active block job', virsh_tries)
+                return [rc, out, err] if rc != 0
+            end
+
+            # Update one:system_datastore metadata tag in running VM
+            tmpf = Tempfile.new([@domain, '.xml'])
+
+            virsh_retry("dumpxml #{@domain} > '#{tmpf.path}'", 'active block job', virsh_tries)
+
+            `sed -i '/one:system_datastore/s:CDATA\\[[^]]*\\]:CDATA[#{dst_dir}]:' '#{tmpf.path}'`
+
+            virsh_retry("define '#{tmpf.path}'", 'active block job', virsh_tries)
+
+            tmpf.unlink
+
+            # `define` makes the VM persistent. We immediately `undefine` it to
+            # keep it transient.
+            undefine
+
+            [0, nil, nil]
+        end
+
         #  Basic domain operations (does not require additional parameters)
         VIRSH_COMMANDS = [
             'resume',
@@ -408,11 +475,6 @@ module VirtualMachineManagerKVM
             @xpath_prefix = ''
         end
 
-        # @return true if the VM includes a PCI device being attached
-        def pci_attach?
-            @xml.exist? "TEMPLATE/PCI[ATTACH='YES']"
-        end
-
         #-----------------------------------------------------------------------
         #  This function generates a XML document to attach a new interface
         #  to the VM. The interface specification supports the same OpenNebula
@@ -438,6 +500,10 @@ module VirtualMachineManagerKVM
             model = env('DEFAULT_ATTACH_NIC_MODEL') if model.empty?
             model.encode!(:xml => :attr) unless model.empty?
 
+            virtio_queues = @xml["#{@xpath_prefix}VIRTIO_QUEUES"]
+            virtio_queues = @xml['TEMPLATE/VCPU'] || '1' if virtio_queues == 'auto'
+            virtio_queues.encode!(:xml => :attr) unless virtio_queues.empty?
+
             filter = @xml["#{@xpath_prefix}FILTER"]
             filter = env('DEFAULT_ATTACH_NIC_FILTER') if filter.empty?
             filter.encode!(:xml => :attr) unless filter.empty?
@@ -461,9 +527,8 @@ module VirtualMachineManagerKVM
             dev << xputs('<boot order=%s/>', 'ORDER')
             dev << "<model type=#{model}/>" unless model.empty?
 
-            if model == 'virtio'
-                dev << xputs('<driver name="vhost" queues=%s/>',
-                             'VIRTIO_QUEUES')
+            if model == '"virtio"' && !virtio_queues.empty?
+                dev << "<driver name='vhost' queues=#{virtio_queues}/>"
             end
 
             if exist?('IP') && !filter.empty?
